/*!
   Copyright 2019 Ron Buckton

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/
/// <reference lib="es2018.asynciterable" />
/**
 * A union of all of the primitive types in TypeScript.
 */
export declare type Primitive = string | symbol | boolean | number | bigint;
/**
 * A union of all of the falsey types in TypeScript.
 */
export declare type Falsey = null | undefined | false | 0 | 0n | '';
/**
 * A PropertyDescriptor constrained to the valid attributes for an accessor.
 */
export interface AccessorPropertyDescriptor<T = any> {
    enumerable?: boolean;
    configurable?: boolean;
    get?(): T;
    set?(v: T): void;
}
/**
 * A PropertyDescriptor constrained to the valid attributes for a method.
 */
export interface MethodPropertyDescriptor<T extends (...args: any[]) => any = (...args: any[]) => any> {
    enumerable?: boolean;
    configurable?: boolean;
    writable?: boolean;
    value: T;
}
/**
 * Indicates a type that may be `undefined`.
 */
export declare type Optional<T> = T | undefined;
/**
 * Strips `undefined` from a type.
 */
export declare type NonOptional<T> = T extends undefined ? never : T;
/**
 * Indicates a type that may be `null` or `undefined`.
 */
export declare type Nullable<T> = T | undefined | null;
/**
 * Strips `null` or `undefined` from a type.
 */
export import NonNullable = globalThis.NonNullable;
/**
 * Gets a union of `keyof T'` of each constituent `T'` of `T`.
 */
export declare type AnyKeyof<T> = T extends unknown ? keyof T : never;
/**
 * Gets a union of `Extract<T', U>` for each constituent `T'` of `T`.
 */
export declare type AnyExtract<T, U> = T extends unknown ? Extract<T, U> : never;
/**
 * Gets a union of `Exclude<T', U>` for each constituent `T'` of `T`.
 */
export declare type AnyExclude<T, U> = T extends unknown ? Exclude<T, U> : never;
/**
 * Represents a concrete ECMAScript constructor object.
 */
export declare type Constructor<T = {}, A extends any[] = any[]> = new (...args: A) => T;
/**
 * Represents an abstract class constructor.
 */
export declare type AbstractConstructor<T = {}> = Function & {
    prototype: T;
};
/**
 * Gets the type yielded by an Iterable.
 */
export declare type IteratedType<T> = T extends {
    [Symbol.iterator](): {
        next(): {
            done: false;
            value: infer U;
        };
    };
} ? U : T extends {
    [Symbol.iterator](): {
        next(): {
            done: true;
        };
    };
} ? never : T extends {
    [Symbol.iterator](): {
        next(): {
            done: boolean;
            value: infer U;
        };
    };
} ? U : T extends {
    [Symbol.iterator](): any;
} ? unknown : never;
/**
 * Gets the type that can be sent to a generator via its `next` method.
 */
export declare type GeneratorNextType<T> = T extends {
    [Symbol.iterator](): {
        next(value?: infer U): any;
    };
} ? U : never;
/**
 * Gets the type that can be returned from a generator when it has finished executing.
 */
export declare type GeneratorReturnType<T> = T extends {
    [Symbol.iterator](): {
        next(): {
            done: true;
            value?: infer U;
        };
    };
} ? U : T extends {
    [Symbol.iterator](): {
        next(): {
            done: false;
        };
    };
} ? never : T extends {
    [Symbol.iterator](): {
        next(): {
            done: boolean;
            value?: infer U;
        };
    };
} ? U : T extends {
    [Symbol.iterator](): any;
} ? void : never;
/**
 * Gets the promised type of a Promise.
 */
export declare type PromisedType<T> = T extends {
    then(onfulfilled: infer U): any;
} ? U extends ((value: infer V) => any) ? V : never : never;
/**
 * Maps to `true` if `A` is precisely the `any` type; otherwise, `false`.
 */
export declare type IsAny<A> = (1 | 2) extends (A extends never ? 1 : 2) ? true : false;
/**
 * Maps to `true` if `A` is precisely the `never` type; otherwise, `false`.
 */
export declare type IsNever<A> = (A extends never ? true : false) extends true ? true : false;
/**
 * Maps to `true` if `A` is precisely the `unknown` type; otherwise, `false`.
 */
export declare type IsUnknown<A> = IsAny<A> extends true ? false : unknown extends A ? true : false;
/**
 * Maps to `true` if `T` is a union of multiple types; otherwise, `false`.
 */
export declare type IsUnion<T> = IsNever<T> extends true ? false : __IsUnionRest<T, [T]>;
declare type __IsUnionRest<T, Q> = T extends unknown ? Not<SameType<[T], Q>> : never;
/**
 * Maps to `true` if `Sub` is a subtype of `Super`; otherwise, `false`.
 */
export declare type IsSubtypeOf<Sub, Super> = IsNever<Super> extends true ? IsNever<Sub> : IsNever<Sub> extends true ? true : IsAny<Super> extends true ? true : IsAny<Sub> extends true ? true : Sub extends Super ? true : false;
/**
 * Maps to `true` if `Super` is a supertype of `Sub`; otherwise, `false`.
 */
export declare type IsSupertypeOf<Super, Sub> = IsSubtypeOf<Sub, Super>;
/**
 * Maps to `true` if the type has a call signature; otherwise, `false`.
 */
export declare type IsCallable<T> = IsAny<T> extends true ? boolean : IsNever<T> extends true ? never : SameType<T, Function> extends true ? true : [T] extends [(...args: any) => any] ? true : false;
/**
 * Maps to `true` if the type has a construct signature; otherwise, `false`.
 */
export declare type IsConstructable<T> = IsAny<T> extends true ? boolean : IsNever<T> extends true ? never : SameType<T, Function> extends true ? true : [T] extends [new (...args: any) => any] ? true : false;
/**
 * Maps to `true` if `A` is `false`, otherwise `true`.
 */
export declare type Not<A extends boolean> = IsNever<A> extends true ? never : A extends false ? true : false;
/**
 * Maps to `true` if both `A` and `B` are `true`; otherwise, `false`.
 */
export declare type And<A extends boolean, B extends boolean> = IsNever<A> extends true ? never : IsNever<B> extends true ? never : A extends false ? false : B extends false ? false : true;
/**
 * Maps to `true` if either `A` or `B` are `true`; otherwise, `false`.
 */
export declare type Or<A extends boolean, B extends boolean> = IsNever<A> extends true ? never : IsNever<B> extends true ? never : A extends true ? true : B extends true ? true : false;
/**
 * Maps to `true` if only one of either `A` or `B` are `true`; otherwise, `false`.
 */
export declare type XOr<A extends boolean, B extends boolean> = IsNever<A> extends true ? never : IsNever<B> extends true ? never : A extends true ? Not<B> : B extends true ? Not<A> : false;
/**
 * Maps to `true` if every element of the tuple `L` is `true`; otherwise, `false`.
 */
export declare type Every<L extends boolean[]> = L extends [] ? never : __EveryRest<{
    [P in keyof L]: IsNever<L[P]> extends true ? "never" : IsAny<L[P]> extends true ? "boolean" : boolean extends L[P] ? "boolean" : L[P] extends false ? "false" : never;
}[number]>;
declare type __EveryRest<R> = "never" extends R ? never : "false" extends R ? false : "boolean" extends R ? boolean : true;
/**
 * Maps to `true` if any element of the tuple `L` is `true`; otherwise, `false`.
 */
export declare type Some<L extends boolean[]> = L extends [] ? never : __SomeRest<{
    [P in keyof L]: IsNever<L[P]> extends true ? "never" : IsAny<L[P]> extends true ? "boolean" : boolean extends L[P] ? "boolean" : L[P] extends true ? "true" : never;
}[number]>;
declare type __SomeRest<R> = "never" extends R ? never : "true" extends R ? true : "boolean" extends R ? boolean : false;
/**
 * Maps to `true` if exactly one element of the tuple `L` is `true`; otherwise, `false`.
 */
export declare type One<L extends boolean[]> = L extends [] ? never : __OneRest<{
    [P in keyof L]: IsNever<L[P]> extends true ? "never" : IsAny<L[P]> extends true ? "boolean" : boolean extends L[P] ? "boolean" : L[P] extends true ? [P] : never;
}[number]>;
declare type __OneRest<R> = "never" extends R ? never : "boolean" extends R ? boolean : IsNever<R> extends true ? false : IsUnion<R> extends true ? false : true;
/**
 * Maps to `true` if both `A` and `B` are assignable to each other; otherwise, `false`.
 */
export declare type SameType<A, B> = IsNever<A> extends true ? IsNever<B> : IsNever<B> extends true ? false : [A, B] extends [B, A] ? true : false;
/**
 * Maps to `true` if all elements of the tuple `L` are assignable to each other; otherwise, `false`.
 */
export declare type SameTypes<L extends any[]> = L extends [] ? never : SameType<{
    [P in keyof L]: SameType<L[P], L[number]>;
}[number], true>;
/**
 * Maps to `true` if either `A` or `B` are relatable to each other.
 */
export declare type Relatable<A, B> = IsNever<A> extends true ? false : IsNever<B> extends true ? false : IsAny<A> extends true ? true : IsAny<B> extends true ? true : [A] extends [B] ? true : [B] extends [A] ? true : false;
/**
 * Maps to `true` if any type in `A` is assignable to any type in `B`; otherwise, `false`.
 */
export declare type Overlaps<A, B> = IsNever<A> extends true ? false : IsNever<B> extends true ? false : IsAny<A> extends true ? true : IsAny<B> extends true ? true : 1 extends (A extends unknown ? A extends B ? 1 : 2 : 3) ? true : 1 extends (B extends unknown ? B extends A ? 1 : 2 : 3) ? true : false;
/**
 * Maps to `true` if `Sub` is a subset of `Super`; otherwise, `false`.
 */
export declare type IsSubsetOf<Sub, Super> = IsAny<Sub> extends true ? boolean : IsAny<Super> extends true ? boolean : __IsSubsetOf<Sub, Super>;
declare type __IsSubsetOf<Sub, Super> = IsNever<Sub> extends true ? true : IsNever<Super> extends true ? false : IsUnknown<Sub> extends true ? false : IsUnknown<Super> extends true ? true : [Sub] extends [Super] ? true : false;
/**
 * Maps to `true` if `Super` is a superset of `Sub`; otherwise, `false`.
 */
export declare type IsSupersetOf<Super, Sub> = IsSubsetOf<Sub, Super>;
/**
 * Maps to `true` if `Sub` is a proper subset of `Super`; otherwise, `false`.
 */
export declare type IsProperSubsetOf<Sub, Super> = IsAny<Sub> extends true ? boolean : IsAny<Super> extends true ? boolean : SameType<Sub, Super> extends true ? false : __IsSubsetOf<Sub, Super>;
/**
 * Maps to `true` if `Super` is a proper superset of `Sub`; otherwise, `false`.
 */
export declare type IsProperSupersetOf<Super, Sub> = IsProperSubsetOf<Sub, Super>;
/**
 * Maps to the keys of `T` whose values match `TMatch`.
 */
export declare type MatchingKeys<T, TMatch> = __MatchingKeys<T, TMatch, keyof T>;
declare type __MatchingKeys<T, TSuper, K extends keyof T> = K extends (T[K] extends TSuper ? K : never) ? K : never;
/**
 * Maps to the keys of `T` whose values do not match `TMatch`.
 */
export declare type NonMatchingKeys<T, TMatch> = Exclude<keyof T, MatchingKeys<T, TMatch>>;
/**
 * Maps to the keys of `T` whose values are functions.
 */
export declare type FunctionKeys<T, F extends Function = Function> = MatchingKeys<T, F>;
/**
 * Maps to the keys of `T` whose values are not functions.
 */
export declare type NonFunctionKeys<T, F extends Function = Function> = NonMatchingKeys<T, F>;
/**
 * Split an object into a union of objects for each key/value pair.
 */
export declare type Disjoin<T extends object> = IsNever<T> extends true ? never : IsAny<T> extends true ? any : __DisjoinRest<{
    [K in keyof T]: {
        [P in K]: T[P];
    };
}[keyof T]>;
declare type __DisjoinRest<T> = IsNever<T> extends true ? {} : T;
/**
 * Map an intersection of object types into a single object type.
 */
export declare type Reshape<T extends object> = Pick<T, keyof T>;
/**
 * Joins a union of disjoint object types into a single object type.
 */
export declare type Conjoin<T extends object> = {
    [P in AnyKeyof<T>]: AnyExtract<T, {
        readonly [U in P]: unknown;
    }>[P];
};
/**
 * Maps to `true` if any type in `A` is assignable to or shares a property with any type in `B`; otherwise, `false`.
 * This is similar to `Overlaps`, except object types in `A` and `B` are mapped through `Disjoin`.
 */
export declare type DisjoinOverlaps<A, B> = Overlaps<A extends object ? Disjoin<A> : A, B extends object ? Disjoin<B> : B>;
/**
 * Maps to `true` if `T` is an empty object (`{}`).
 */
export declare type IsEmpty<T extends object> = IsNever<keyof T>;
/**
 * Remove from `T` all keys in `K`.
 */
export declare type Omit<T, K extends PropertyKey> = Pick<T, Exclude<keyof T, K>>;
/**
 * Remove from `A` all properties with the same types that exist in `B`.
 */
export declare type Diff<A extends object, B extends object> = Omit<A, keyof B>;
/**
 * Pick from `A` all properties with the same types that exist in `B`.
 */
export declare type Intersect<A extends object, B extends object> = Pick<A & B, Extract<keyof A, keyof B> & Extract<keyof B, keyof A>>;
/**
 * Combine the properties of `A` and `B`, chosing the properties in `B` if the types differ.
 */
export declare type Assign<A extends object, B extends object> = Reshape<Diff<A, B> & B>;
/**
 * Maps to a mutable copy of T.
 */
export declare type Mutable<T> = {
    -readonly [P in keyof T]: T[P];
};
export {};
//# sourceMappingURL=index.d.ts.map